{{/*
Licensed to the Apache Software Foundation (ASF) under one or more
contributor license agreements.  See the NOTICE file distributed with
this work for additional information regarding copyright ownership.
The ASF licenses this file to You under the Apache License, Version 2.0
(the "License"); you may not use this file except in compliance with
the License.  You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/}}

{{- define "configmap-content-no-seed" -}}
{{- /*
  configmap content without seed list.
*/ -}}
{{- $root := . }}
{{- $values := .Values }}

{{- /*
  It's impossible to do a rolling upgrade from not-tls-enabled rpc to tls-enabled rpc.
*/ -}}
{{- $check := list
    (include "redpanda-atleast-23-1-2" .|fromJson).bool
    (include "redpanda-22-3-atleast-22-3-13" .|fromJson).bool
    (include "redpanda-22-2-atleast-22-2-10" .|fromJson).bool
-}}
{{- $wantedRPCTLS := (include "rpc-tls-enabled" . | fromJson).bool -}}
{{- if and (not (mustHas true $check)) $wantedRPCTLS -}}
  {{- fail (printf "Redpanda version v%s does not support TLS on the RPC port. Please upgrade. See technical service bulletin 2023-01." (include "redpanda.semver" .)) -}}
{{- end -}}
{{- $cm := lookup "v1" "ConfigMap" .Release.Namespace (include "redpanda.fullname" .) -}}
{{- $redpandaYAML := dig "data" "redpanda.yaml" "" $cm | fromYaml -}}
{{- $currentRPCTLS := dig "redpanda" "rpc_server_tls" "enabled" false $redpandaYAML -}}
{{- /* Lookup will return an empty map when running `helm template` or when `--dry-run` is passed. */ -}}
{{- if (and .Release.IsUpgrade $cm) -}}
  {{- if ne $currentRPCTLS $wantedRPCTLS -}}
    {{- if eq (get .Values "force" | default false) false -}}
      {{- fail (join "\n" (list
          (printf "\n\nError: Cannot do a rolling restart to enable or disable tls at the RPC layer: changing listeners.rpc.tls.enabled (redpanda.yaml:repdanda.rpc_server_tls.enabled) from %v to %v" $currentRPCTLS $wantedRPCTLS)
          "***WARNING The following instructions will result in a short period of downtime."
          "To accept this risk, run the upgrade again adding `--force=true` and do the following:\n"
          "While helm is upgrading the release, manually delete ALL the pods:"
          (printf "    kubectl -n %s delete pod -l app.kubernetes.io/component=redpanda-statefulset" .Release.Namespace)
          "\nIf you got here thinking rpc tls was already enabled, see technical service bulletin 2023-01."
          ))
      -}}
    {{- end -}}
  {{- end -}}
{{- end -}}

{{- $users := list -}}
{{- if (include "sasl-enabled" . | fromJson).bool -}}
  {{- range $user := .Values.auth.sasl.users -}}
    {{- $users = append $users $user.name -}}
  {{- end -}}
{{- end -}}

  bootstrap.yaml: |
    kafka_enable_authorization: {{ (include "sasl-enabled" . | fromJson).bool }}
    enable_sasl: {{ (include "sasl-enabled" . | fromJson).bool }}
    enable_rack_awareness: {{ .Values.rackAwareness.enabled }}
  {{- if $users }}
    superusers: {{ toJson $users }}
  {{- end }}
  {{- with (dig "cluster" dict .Values.config) }}
      {{- range $key, $element := .}}
        {{- if or (eq (typeOf $element) "bool") $element }}
    {{ $key }}: {{ $element | toYaml }}
        {{- end }}
      {{- end }}
  {{- end }}
  {{- include "tunable" . }}
  {{- if and (not (hasKey .Values.config.cluster "storage_min_free_bytes")) ((include "redpanda-atleast-22-2-0" . | fromJson).bool) }}
    storage_min_free_bytes: {{ include "storage-min-free-bytes" . }}
  {{- end }}
{{- if and (include "is-licensed" . | fromJson).bool .Values.storage.tieredConfig.cloud_storage_enabled }}
  {{- $tieredStorageConfig := deepCopy .Values.storage.tieredConfig }}
  {{- $tieredStorageConfig = unset $tieredStorageConfig "cloud_storage_cache_directory" }}
  {{- if not (include "redpanda-atleast-22-3-0" . | fromJson).bool }}
    {{- $tieredStorageConfig = unset $tieredStorageConfig "cloud_storage_credentials_source"}}
  {{- end }}
  {{- range $key, $element := $tieredStorageConfig}}
    {{- if or (eq (typeOf $element) "bool") $element }}
    {{ $key }}: {{ $element | toYaml }}
    {{- end }}
  {{- end }}
{{- end }}
  redpanda.yaml: |
    config_file: /etc/redpanda/redpanda.yaml
{{- if .Values.logging.usageStats.enabled }}
  {{- with (dig "usageStats" "organization" "" .Values.logging) }}
    organization: {{ . }}
  {{- end }}
  {{- with (dig "usageStats" "clusterId" "" .Values.logging) }}
    cluster_id: {{ . }}
  {{- end }}
{{- end }}
    redpanda:
{{- if (include "redpanda-atleast-22-3-0" . | fromJson).bool }}
      empty_seed_starts_cluster: false
{{- end }}
      kafka_enable_authorization: {{ (include "sasl-enabled" . | fromJson).bool }}
      enable_sasl: {{ (include "sasl-enabled" . | fromJson).bool }}
  {{- if $users }}
      superusers: {{ toJson $users }}
  {{- end }}
  {{- with (dig "cluster" dict .Values.config) }}
      {{- range $key, $element := . }}
        {{- if or (eq (typeOf $element) "bool") $element }}
      {{ $key }}: {{ $element | toYaml }}
        {{- end }}
      {{- end }}
  {{- end }}
  {{- with (dig "tunable" dict .Values.config) }}
      {{- range $key, $element := .}}
        {{- if or (eq (typeOf $element) "bool") $element }}
      {{ $key }}: {{ $element | toYaml }}
        {{- end }}
      {{- end }}
  {{- end }}
  {{- if not (hasKey .Values.config.cluster "storage_min_free_bytes") }}
      storage_min_free_bytes: {{ include "storage-min-free-bytes" . }}
  {{- end }}
      {{- with dig "node" dict .Values.config }}
      {{- range $key, $element := .}}
        {{- if and (or (eq (typeOf $element) "bool") $element) (and (eq $key "crash_loop_limit") (include "redpanda-atleast-23-1-1" $root | fromJson).bool) }}
      {{ $key }}: {{ $element | toYaml }}
        {{- end }}
      {{- end }}
      {{- end }}
{{- /* LISTENERS */}}
{{- /* Admin API */}}
{{- $service := .Values.listeners.admin }}
      admin:
        - name: internal
          address: 0.0.0.0
          port: {{ $service.port }}
{{- range $name, $listener := $service.external }}
{{- if and $listener.port $name }}
        - name: {{ $name }}
          address: 0.0.0.0
          port: {{ $listener.port }}
{{- end }}
{{- end }}
      admin_api_tls:
{{- if (include "admin-internal-tls-enabled" . | fromJson).bool }}
        - name: internal
          enabled: true
          cert_file: /etc/tls/certs/{{ $service.tls.cert }}/tls.crt
          key_file: /etc/tls/certs/{{ $service.tls.cert }}/tls.key
          require_client_auth: {{ $service.tls.requireClientAuth }}
  {{- $cert := get .Values.tls.certs $service.tls.cert }}
  {{- if empty $cert }}
    {{- fail (printf "Certificate, '%s', used but not defined")}}
  {{- end }}
  {{- if $cert.caEnabled }}
          truststore_file: /etc/tls/certs/{{ $service.tls.cert }}/ca.crt
  {{- else }}
        {{- /* This is a required field so we use the default in the redpanda debian container */}}
          truststore_file: /etc/ssl/certs/ca-certificates.crt
  {{- end }}
{{- end }}
{{- range $name, $listener := $service.external }}
  {{- $k := dict "Values" $values "listener" $listener }}
  {{- if (include "admin-external-tls-enabled" $k | fromJson).bool }}
    {{- $mtls := dig "tls" "requireClientAuth" false $listener }}
    {{- $mtls = dig "tls" "requireClientAuth" $mtls $k }}
    {{- $certName := include "admin-external-tls-cert" $k }}
    {{- $certPath := printf "/etc/tls/certs/%s" $certName }}
    {{- $cert := get $values.tls.certs $certName }}
    {{- if empty $cert }}
      {{- fail (printf "Certificate, '%s', used but not defined" $certName)}}
    {{- end }}
        - name: {{ $name }}
          enabled: true
          cert_file: {{ $certPath }}/tls.crt
          key_file: {{ $certPath }}/tls.key
          require_client_auth: {{ $mtls }}
    {{- if $cert.caEnabled }}
          truststore_file: {{ $certPath }}/ca.crt
    {{- else }}
          {{- /* This is a required field so we use the default in the redpanda debian container */}}
          truststore_file: /etc/ssl/certs/ca-certificates.crt
    {{- end }}
  {{- end }}
{{- end }}
{{- /* Kafka API */}}
{{- $kafkaService := .Values.listeners.kafka }}
      kafka_api:
        - name: internal
          address: 0.0.0.0
          port: {{ $kafkaService.port }}
          {{- if or (include "sasl-enabled" $root | fromJson).bool $kafkaService.authenticationMethod }}
          authentication_method: {{ default "sasl" $kafkaService.authenticationMethod }}
          {{- end }}
{{- range $name, $listener := $kafkaService.external }}
        - name: {{ $name }}
          address: 0.0.0.0
          port: {{ $listener.port }}
          {{- if or (include "sasl-enabled" $root | fromJson).bool $listener.authenticationMethod }}
          authentication_method: {{ default "sasl" $listener.authenticationMethod }}
          {{- end }}
{{- end }}
      kafka_api_tls:
{{- if (include "kafka-internal-tls-enabled" . | fromJson).bool }}
        - name: internal
          enabled: true
          cert_file: /etc/tls/certs/{{ $kafkaService.tls.cert }}/tls.crt
          key_file: /etc/tls/certs/{{ $kafkaService.tls.cert }}/tls.key
          require_client_auth: {{ $kafkaService.tls.requireClientAuth }}
  {{- $cert := get .Values.tls.certs $kafkaService.tls.cert }}
  {{- if empty $cert }}
    {{- fail (printf "Certificate, '%s', used but not defined")}}
  {{- end }}
  {{- if $cert.caEnabled }}
          truststore_file: /etc/tls/certs/{{ $kafkaService.tls.cert }}/ca.crt
  {{- else }}
          {{- /* This is a required field so we use the default in the redpanda debian container */}}
          truststore_file: /etc/ssl/certs/ca-certificates.crt
  {{- end }}
{{- end }}
{{- range $name, $listener := $kafkaService.external }}
  {{- $k := dict "Values" $values "listener" $listener }}
  {{- if (include "kafka-external-tls-enabled" $k | fromJson).bool }}
    {{- $mtls := dig "tls" "requireClientAuth" false $listener }}
    {{- $mtls = dig "tls" "requireClientAuth" $mtls $k }}
    {{- $certName := include "kafka-external-tls-cert" $k }}
    {{- $certPath := printf "/etc/tls/certs/%s" $certName }}
    {{- $cert := get $values.tls.certs $certName }}
    {{- if empty $cert }}
      {{- fail (printf "Certificate, '%s', used but not defined" $certName)}}
    {{- end }}
        - name: {{ $name }}
          enabled: true
          cert_file: {{ $certPath }}/tls.crt
          key_file: {{ $certPath }}/tls.key
          require_client_auth: {{ $mtls }}
    {{- if $cert.caEnabled }}
          truststore_file: {{ $certPath }}/ca.crt
    {{- else }}
          {{- /* This is a required field so we use the default in the redpanda debian container */}}
          truststore_file: /etc/ssl/certs/ca-certificates.crt
    {{- end }}
  {{- end }}
{{- end }}
{{- /* RPC Server */}}
{{- $service = .Values.listeners.rpc }}
      rpc_server:
        address: 0.0.0.0
        port: {{ $service.port }}
{{- if (include "rpc-tls-enabled" . | fromJson).bool }}
      rpc_server_tls:
        enabled: true
        cert_file: /etc/tls/certs/{{ $service.tls.cert }}/tls.crt
        key_file: /etc/tls/certs/{{ $service.tls.cert }}/tls.key
        require_client_auth: {{ $service.tls.requireClientAuth }}
  {{- $cert := get .Values.tls.certs $service.tls.cert }}
  {{- if empty $cert }}
    {{- fail (printf "Certificate, '%s', used but not defined")}}
  {{- end }}
  {{- if $cert.caEnabled }}
        truststore_file: /etc/tls/certs/{{ $service.tls.cert }}/ca.crt
  {{- else }}
        {{- /* This is a required field so we use the default in the redpanda debian container */}}
        truststore_file: /etc/ssl/certs/ca-certificates.crt
  {{- end }}
{{- end }}
      seed_servers:
{{- with $root.tempConfigMapServerList -}}
  {{- . | trim | nindent 8 }}
{{- end -}}
{{- if and (include "is-licensed" . | fromJson).bool .Values.storage.tieredConfig.cloud_storage_enabled }}
  {{- $tieredStorageConfig := deepCopy .Values.storage.tieredConfig }}
  {{- if not (include "redpanda-atleast-22-3-0" . | fromJson).bool }}
    {{- $tieredStorageConfig = unset $tieredStorageConfig "cloud_storage_credentials_source"}}
  {{- end }}
  {{- range $key, $element := $tieredStorageConfig}}
    {{- if or (eq (typeOf $element) "bool") $element }}
    {{ $key }}: {{ $element | toYaml }}
    {{- end }}
  {{- end }}
{{- end }}
{{- /* Schema Registry API */}}
{{- if and .Values.listeners.schemaRegistry.enabled (include "redpanda-22-2-x-without-sasl" $root | fromJson).bool }}
  {{- $schemaRegistryService := .Values.listeners.schemaRegistry }}
    schema_registry:
      schema_registry_api:
        - name: internal
          address: 0.0.0.0
          port: {{ $schemaRegistryService.port }}
          {{- if or (include "sasl-enabled" $root | fromJson).bool $schemaRegistryService.authenticationMethod }}
          authentication_method: {{ default "http_basic" $schemaRegistryService.authenticationMethod }}
          {{- end }}
  {{- range $name, $listener := $schemaRegistryService.external }}
        - name: {{ $name }}
          address: 0.0.0.0
          {{- /*
            when upgrading from an older version that had a missing port, fail if we cannot guess a default
            this should work in all cases as the older versions would have failed with multiple listeners anyway
          */}}
          {{- if and (empty $listener.port) (ne (len $schemaRegistryService.external) 1) }}
            {{- fail "missing required port for schemaRegistry listener $listener.name" }}
          {{- end }}
          port: {{ $listener.port | default 8084 }}
          {{- if or (include "sasl-enabled" $root | fromJson).bool $listener.authenticationMethod }}
          authentication_method: {{ default "http_basic" $listener.authenticationMethod }}
          {{- end }}
  {{- end }}
      schema_registry_api_tls:
  {{- if (include "schemaRegistry-internal-tls-enabled" . | fromJson).bool }}
        - name: internal
          enabled: true
          cert_file: /etc/tls/certs/{{ $schemaRegistryService.tls.cert }}/tls.crt
          key_file: /etc/tls/certs/{{ $schemaRegistryService.tls.cert }}/tls.key
          require_client_auth: {{ $schemaRegistryService.tls.requireClientAuth }}
    {{- $cert := get .Values.tls.certs $schemaRegistryService.tls.cert }}
    {{- if empty $cert }}
      {{- fail (printf "Certificate, '%s', used but not defined")}}
    {{- end }}
    {{- if $cert.caEnabled }}
          truststore_file: /etc/tls/certs/{{ $schemaRegistryService.tls.cert }}/ca.crt
    {{- else }}
          {{- /* This is a required field so we use the default in the redpanda debian container */}}
          truststore_file: /etc/ssl/certs/ca-certificates.crt
    {{- end }}
  {{- end }}
  {{- range $name, $listener := $schemaRegistryService.external }}
    {{- $k := dict "Values" $values "listener" $listener }}
    {{- if (include "schemaRegistry-external-tls-enabled" $k | fromJson).bool }}
      {{- $mtls := dig "tls" "requireClientAuth" false $listener }}
      {{- $mtls = dig "tls" "requireClientAuth" $mtls $k }}
      {{- $certName := include "schemaRegistry-external-tls-cert" $k }}
      {{- $certPath := printf "/etc/tls/certs/%s" $certName }}
      {{- $cert := get $values.tls.certs $certName }}
      {{- if empty $cert }}
        {{- fail (printf "Certificate, '%s', used but not defined")}}
      {{- end }}
        - name: {{ $name }}
          enabled: true
          cert_file: {{ $certPath }}/tls.crt
          key_file: {{ $certPath }}/tls.key
          require_client_auth: {{ $mtls }}
      {{- if $cert.caEnabled }}
          truststore_file: {{ $certPath }}/ca.crt
      {{- else }}
          {{- /* This is a required field so we use the default in the redpanda debian container */}}
          truststore_file: /etc/ssl/certs/ca-certificates.crt
      {{- end }}
    {{- end }}
  {{- end }}
{{- end }}
{{- /* HTTP Proxy */}}
{{- if and .Values.listeners.http.enabled (include "redpanda-22-2-x-without-sasl" $root | fromJson).bool }}
  {{- $HTTPService := .Values.listeners.http }}
    pandaproxy:
      pandaproxy_api:
        - name: internal
          address: 0.0.0.0
          port: {{ $HTTPService.port }}
          {{- if or (include "sasl-enabled" $root | fromJson).bool $HTTPService.authenticationMethod }}
          authentication_method: {{ default "http_basic" $HTTPService.authenticationMethod }}
          {{- end }}
  {{- range $name, $listener := $HTTPService.external }}
        - name: {{ $name }}
          address: 0.0.0.0
          port: {{ $listener.port }}
          {{- if or (include "sasl-enabled" $root | fromJson).bool $listener.authenticationMethod }}
          authentication_method: {{ default "http_basic" $listener.authenticationMethod }}
          {{- end }}
  {{- end }}
      pandaproxy_api_tls:
  {{- if (include "http-internal-tls-enabled" . | fromJson).bool }}
        - name: internal
          enabled: true
          cert_file: /etc/tls/certs/{{ $HTTPService.tls.cert }}/tls.crt
          key_file: /etc/tls/certs/{{ $HTTPService.tls.cert }}/tls.key
          require_client_auth: {{ $HTTPService.tls.requireClientAuth }}
    {{- $cert := get .Values.tls.certs $HTTPService.tls.cert }}
    {{- if empty $cert }}
      {{- fail (printf "Certificate, '%s', used but not defined")}}
    {{- end }}
    {{- if $cert.caEnabled }}
          truststore_file: /etc/tls/certs/{{ $HTTPService.tls.cert }}/ca.crt
    {{- else }}
          {{- /* This is a required field so we use the default in the redpanda debian container */}}
          truststore_file: /etc/ssl/certs/ca-certificates.crt
    {{- end }}
  {{- end }}
  {{- range $name, $listener := $HTTPService.external }}
    {{- $k := dict "Values" $values "listener" $listener }}
    {{- if (include "http-external-tls-enabled" $k | fromJson).bool }}
      {{- $mtls := dig "tls" "requireClientAuth" false $listener }}
      {{- $mtls = dig "tls" "requireClientAuth" $mtls $k }}
      {{- $certName := include "http-external-tls-cert" $k }}
      {{- $certPath := printf "/etc/tls/certs/%s" $certName }}
      {{- $cert := get $values.tls.certs $certName }}
      {{- if empty $cert }}
        {{- fail (printf "Certificate, '%s', used but not defined")}}
      {{- end }}
        - name: {{ $name }}
          enabled: true
          cert_file: {{ $certPath }}/tls.crt
          key_file: {{ $certPath }}/tls.key
          require_client_auth: {{ $mtls }}
      {{- if $cert.caEnabled }}
          truststore_file: {{ $certPath }}/ca.crt
      {{- else }}
          {{- /* This is a required field so we use the default in the redpanda debian container */}}
          truststore_file: /etc/ssl/certs/ca-certificates.crt
      {{- end }}
    {{- end }}
  {{- end }}
{{- end }}
{{- /* END LISTENERS */}}

    rpk:
{{- with (dig "rpk" dict .Values.config) }}
  {{- . | toYaml | nindent 6}}
{{- end }}
      enable_usage_stats: {{ .Values.logging.usageStats.enabled }}
      overprovisioned: {{ dig "cpu" "overprovisioned" false .Values.resources }}
      enable_memory_locking: {{ dig "memory" "enable_memory_locking" false .Values.resources }}
{{- if hasKey .Values.tuning "tune_aio_events" }}
      tune_aio_events: {{ .Values.tuning.tune_aio_events }}
{{- end }}
{{- if hasKey .Values.tuning "tune_clocksource" }}
      tune_clocksource: {{ .Values.tuning.tune_clocksource }}
{{- end }}
{{- if hasKey .Values.tuning "tune_ballast_file" }}
      tune_ballast_file: {{ .Values.tuning.tune_ballast_file }}
{{- end }}
{{- if hasKey .Values.tuning "ballast_file_path" }}
      ballast_file_path: {{ .Values.tuning.ballast_file_path }}
{{- end }}
{{- if hasKey .Values.tuning "ballast_file_size" }}
      ballast_file_size: {{ .Values.tuning.ballast_file_size }}
{{- end }}
{{- if hasKey .Values.tuning "well_known_io" }}
      well_known_io: {{ .Values.tuning.well_known_io }}
{{- end }}
{{- end -}}

{{- define "configmap-server-list" -}}
{{- $root := . }}
{{- range (include "seed-server-list" $root | mustFromJson) }}
- host:
    address: {{ . }}
    port: {{  $root.Values.listeners.rpc.port }}
{{- end }}
{{- end -}}

{{- define "configmap-with-server-list" -}}
{{- $root := . }}
{{- $serverList := (include "configmap-server-list" $root ) -}}
{{- $r := set $root "tempConfigMapServerList" ( $serverList ) }}
{{ include "configmap-content-no-seed" $r  }}
{{- end -}}